package com.jtxt.demo.controller;

import com.jtxt.demo.model.Exam;
import com.jtxt.demo.model.LearningMaterial;
import com.jtxt.demo.model.Question;
import com.jtxt.demo.model.TagToResources;
import com.jtxt.demo.repository.TagToResourcesRepository;
import com.jtxt.demo.service.ExamService;
import com.jtxt.demo.service.LearningMaterialService;
import com.jtxt.demo.service.QuestionService;
import com.jtxt.demo.service.TagToResourcesService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
public class TagController {
    @Autowired
    QuestionService questionService;
    @Autowired
    TagToResourcesService tagToResourcesService;
    @Autowired
    LearningMaterialService materialService;
    @Autowired
    ExamService examService;
    @Autowired
    TagToResourcesRepository tagToResourcesRepository;

    @GetMapping("/tags/{tag}")
    public ResponseEntity<?> getTag(@PathVariable String tag) {
        TagToResources tagResult = tagToResourcesService.findResourcesByTag(tag);
        if (tagResult == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(tagResult, HttpStatus.OK);
    }

    @PostMapping("/tags/{tag}")
    public ResponseEntity<?> addToTag(@PathVariable String tag,
                                       @RequestParam(value = "material", required = false) String materialId,
                                       @RequestParam(value = "question", required = false) String questionId,
                                       @RequestParam(value = "exam", required = false) String examId) {
        if (materialId == null && questionId == null) {
            return new ResponseEntity<>(HttpStatus.NOT_ACCEPTABLE);
        }
        if (materialId != null) {
            LearningMaterial material = materialService.getMaterialById(materialId);
            if (material == null) {
                return new ResponseEntity<>(HttpStatus.NOT_FOUND);
            }
            tagToResourcesService.addMaterialToTag(tag, materialId);
        }
        if (questionId != null) {
            Question question = questionService.findQuestionById(questionId);
            if (question == null) {
                return new ResponseEntity<>(HttpStatus.NOT_FOUND);
            }
            tagToResourcesService.addQuestionToTag(tag, questionId);
        }
        if (examId != null) {
            Exam exam = examService.getExamById(examId);
            if (exam == null) {
                return new ResponseEntity<>(HttpStatus.NOT_FOUND);
            }

        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @DeleteMapping("/tags/{tag}")
    public ResponseEntity<?> removeFromTag(@PathVariable String tag,
                                          @RequestParam(value = "material", required = false) String materialId,
                                          @RequestParam(value = "question", required = false) String questionId,
                                          @RequestParam(value = "exam", required = false) String examId) {
        if (materialId == null && questionId == null) {
            return new ResponseEntity<>(HttpStatus.NOT_ACCEPTABLE);
        }
        if (materialId != null) {
            tagToResourcesService.removeMaterialFromTag(tag, materialId);
        }
        if (questionId != null) {
            tagToResourcesService.removeQuestionFromTag(tag, questionId);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @DeleteMapping("/admin/tags")
    public ResponseEntity<?> removeAllTags() {
        tagToResourcesRepository.deleteAll();
        return new ResponseEntity<>(HttpStatus.OK);
    }
}
