package com.jtxt.demo.service;

import com.jtxt.demo.model.LeaderBoard;
import com.jtxt.demo.model.User;
import com.jtxt.demo.repository.LeaderBoardRepository;
import com.jtxt.demo.util.CacheStore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Comparator;
import java.util.stream.Collectors;

import static com.jtxt.demo.util.Utils.CHINA_ZONE;

@Service
public class LeaderBoardServiceImpl implements LeaderBoardService {
    @Autowired
    LeaderBoardRepository leaderBoardRepository;
    @Override
    public LeaderBoard findByName(String name) {
        return leaderBoardRepository.findByName(name).orElse(null);
    }

    @Autowired
    private CacheStore<LeaderBoard> leaderBoardCacheStore;

    @Autowired
    private UserService userService;

    @Override
    public LeaderBoard getLatestLeaderBoardByType(LeaderBoard.LeaderBoardType type) {
        LeaderBoard leaderBoardInCache = leaderBoardCacheStore.get(type.name());
        if (leaderBoardInCache != null) {
            return leaderBoardInCache;
        }
        switch(type) {
            case YuejinBang:
                break;
            case ZongBang:
                LeaderBoard leaderBoard = getZongBang();
                leaderBoardCacheStore.add(LeaderBoard.LeaderBoardType.ZongBang.name(), leaderBoard);
                return leaderBoard;
            case ShiGeBang:
                break;
        }
        return null;
    }

    private LeaderBoard getZongBang() {
        LeaderBoard leaderBoard = new LeaderBoard();
        leaderBoard.setUsers(userService.findAllUsers().stream().sorted(Comparator.comparing(User::getPoints).reversed()).collect(Collectors.toList()));
        leaderBoard.setCreatedTime(LocalDateTime.now(CHINA_ZONE));
        leaderBoard.setType(LeaderBoard.LeaderBoardType.ZongBang);
        return leaderBoard;
    }

}
